#pragma once
#include "opencv2/imgproc.hpp"
#include "opencv2/calib3d.hpp"

struct Vector3
{
	float data[3];

	Vector3 operator-() const
	{
		Vector3 v = { -data[0], -data[1], -data[2] };
		return v;
	}
};

class Marker
{
public:
	std::vector<cv::Point> points;
	int id;
	cv::Mat perspM;
	cv::Mat perspMInv;

	cv::Matx33f rotMatrix;
	Vector3 translation;
};

class CameraCalibration
{
public:
	CameraCalibration();
	cv::Mat1f m_intrinsic;
	std::vector<float> m_distorsion;
};

void performThreshold(const cv::Mat& grayscale, cv::Mat& thresholdImg);
void findContours(const cv::Mat& thresholdImg,
	std::vector< std::vector<cv::Point> >& contours,
	int minContourPointsAllowed);
void findMarkerQuads(std::vector< std::vector<cv::Point> >& contours,
	std::vector<Marker>& markers,
	float minContourLengthAllowed);
void removeMarkerDuplicates(std::vector<Marker>& markersIn,
	std::vector<Marker>& markersOut);
void removePerspective(const cv::Mat &image, std::vector<Marker>& markers, std::vector<cv::Mat>& canonicalMarkers);
void getMarkerIds(std::vector<cv::Mat>& canonicalMarkers, std::vector<Marker>& markers);
void estimatePosition(std::vector<Marker>& detectedMarkers, CameraCalibration& calibration);