#include "opencv2/imgcodecs.hpp"
#include "opencv2/imgproc.hpp"
#include "opencv2/videoio.hpp"
#include <opencv2/highgui.hpp>
#include <opencv2/video.hpp>
#include "MarkerBasedAR.h"

using namespace cv;

const bool imageReplace = true;

int main(int argc, char* argv[])
{
	Mat myImage;
	VideoCapture capture;
	if (argc < 2)
	{
		capture.open(1);
	}
	else
	{
		capture.open(argv[1]);
		capture.set(CAP_PROP_FRAME_WIDTH, 640);
		capture.set(CAP_PROP_FRAME_HEIGHT, 480);
		if (argc > 2)
		{
			myImage = cv::imread(argv[2]);
		}
	}

	namedWindow("PreviewWindow");

	Mat frame;
	while (capture.read(frame))
	{
		Mat resized;
		resize(frame, resized, Size(640, frame.rows * frame.cols / 640.0));
		Mat gray;
		cvtColor(resized, gray, CV_BGR2GRAY);
		Mat bw;
		performThreshold(gray, bw);

		std::vector< std::vector<cv::Point> > contours;
		findContours(bw, contours, 100);
		
		std::vector< Marker > markers1;
		findMarkerQuads(contours, markers1, 1);

		std::vector< Marker > markers2;
		removeMarkerDuplicates(markers1, markers2);

		Mat contourImage = Mat::zeros(bw.rows, bw.cols, CV_8UC3);
		std::vector< std::vector<cv::Point> > contoursToDraw;
		for (int i = 0; i < markers2.size(); ++i)
			contoursToDraw.push_back(markers2[i].points);
		for (int i = 0; i < markers2.size(); ++i)
			drawContours(contourImage, contoursToDraw, i, Scalar(255, 255, 255));	
		//imshow("PreviewWindow", contourImage);

		std::vector<cv::Mat> canonicalMarkers;
		removePerspective(gray, markers2, canonicalMarkers);
		
		for (size_t i = 0; i < canonicalMarkers.size(); ++i)
		{
			cv::threshold(canonicalMarkers[i], canonicalMarkers[i], 125, 255, cv::THRESH_BINARY |
				cv::THRESH_OTSU);
		}

		// Debug
		//for (size_t i = 0; i < canonicalMarkers.size(); ++i)
		//{
		//	std::stringstream sstr;
		//	sstr << "marker_" << i;
		//	imshow(sstr.str(), canonicalMarkers[i]);
		//}

		getMarkerIds(canonicalMarkers, markers2);
		estimatePosition(markers2, CameraCalibration());
		for (int i = 0; i < markers2.size(); ++i)
		{
			printf("Marker %i id: %i\n", i, markers2[i].id);
			printf("Marker %i pos: %f %f %f\n", i, markers2[i].translation.data[0], markers2[i].translation.data[1], markers2[i].translation.data[2]);
		}
		
		if(imageReplace)
		{
			cv::Mat final = resized.clone();
			for (int i = 0; i < markers2.size(); ++i)
			{
				//if (markers2[i].id == -1)
				//	continue;
				double m[3][3] = { { (double)canonicalMarkers[i].cols / myImage.cols, 0, 0 },
				{ 0, (double)canonicalMarkers[i].rows / myImage.rows, 0 },
				{ 0, 0, 1 } };
				Mat M = Mat(3, 3, CV_64F, m);
				M = markers2[i].perspMInv * M;
				cv::Mat layer;
				cv::warpPerspective(myImage, layer, M, cv::Size(resized.cols, resized.rows));
				for (int x = 0; x < resized.cols; ++x)
					for (int y = 0; y < resized.rows; ++y)
					{
					final.at<cv::Vec3b>(y, x) = layer.at<cv::Vec3b>(y, x)[0] == 0 ? final.at<cv::Vec3b>(y, x) : layer.at<cv::Vec3b>(y, x);
					}
				//waitKey();
			}
			imshow("PreviewWindow", final);
		}
		
		char key = static_cast<char>(waitKey(1));
		if (key == 27) { printf("exit"); break; }
	}
	waitKey();
}